package cn.com.infosec.crypto.macs;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.Mac;
import cn.com.infosec.crypto.modes.CBCBlockCipher;
import cn.com.infosec.crypto.paddings.ISO7816d4Padding;

public class CMac implements Mac {
   private static final byte CONSTANT_128 = -121;
   private static final byte CONSTANT_64 = 27;
   private byte[] ZEROES;
   private byte[] mac;
   private byte[] buf;
   private int bufOff;
   private BlockCipher cipher;
   private int macSize;
   private byte[] L;
   private byte[] Lu;
   private byte[] Lu2;

   public CMac(BlockCipher cipher) {
      this(cipher, cipher.getBlockSize() * 8);
   }

   public CMac(BlockCipher cipher, int macSizeInBits) {
      if (macSizeInBits % 8 == 0) {
         if (macSizeInBits <= cipher.getBlockSize() * 8) {
            if (cipher.getBlockSize() != 8 && cipher.getBlockSize() != 16) {
               throw new IllegalArgumentException("Block size must be either 64 or 128 bits");
            } else {
               this.cipher = new CBCBlockCipher(cipher);
               this.macSize = macSizeInBits / 8;
               this.mac = new byte[cipher.getBlockSize()];
               this.buf = new byte[cipher.getBlockSize()];
               this.ZEROES = new byte[cipher.getBlockSize()];
               this.bufOff = 0;
            }
         } else {
            throw new IllegalArgumentException("MAC size must be less or equal to " + cipher.getBlockSize() * 8);
         }
      } else {
         throw new IllegalArgumentException("MAC size must be multiple of 8");
      }
   }

   private final byte[] doubleLu(byte[] in) {
      int var4 = (in[0] & 255) >> 7;
      byte[] var2 = new byte[in.length];

      for(int var3 = 0; var3 < in.length - 1; var2[var3] = (byte)((in[var3] << 1) + ((in[++var3] & 255) >> 7))) {
      }

      int var10000 = var4;
      var4 = in.length - 1;
      var2[var4] = (byte)(in[in.length - 1] << 1);
      if (var10000 == 1) {
         byte in1 = var2[var4 = in.length - 1];
         byte var6;
         if (in.length == 16) {
            var6 = -121;
         } else {
            var6 = 27;
         }

         var2[var4] = (byte)(in1 ^ var6);
      }

      return var2;
   }

   public String getAlgorithmName() {
      return this.cipher.getAlgorithmName();
   }

   public void init(CipherParameters params) {
      CMac var10000 = this;
      CMac var10001 = this;
      CMac var10002 = this;
      CMac var10003 = this;
      CMac var10004 = this;
      CMac var10005 = this;
      CMac var10006 = this;
      this.reset();
      this.cipher.init(true, params);
      byte[] var2;
      byte[] var3;
      this.L = var2 = new byte[(var3 = this.ZEROES).length];
      var10006.cipher.processBlock(var3, 0, var2, 0);
      var10003.Lu = var3 = var10004.doubleLu(var10005.L);
      var10001.Lu2 = var10002.doubleLu(var3);
      var10000.cipher.init(true, params);
   }

   public int getMacSize() {
      return this.macSize;
   }

   public void update(byte in) {
      byte[] var2;
      if (this.bufOff == (var2 = this.buf).length) {
         this.cipher.processBlock(var2, 0, this.mac, 0);
         this.bufOff = 0;
      }

      byte[] var10000 = this.buf;
      int var3;
      this.bufOff = (var3 = this.bufOff) + 1;
      var10000[var3] = in;
   }

   public void update(byte[] in, int inOff, int len) {
      if (len < 0) {
         throw new IllegalArgumentException("Can't have a negative input length!");
      } else {
         int var4;
         int var5;
         int var6;
         int var10000;
         if (len > (var6 = (var4 = this.cipher.getBlockSize()) - (var5 = this.bufOff))) {
            var10000 = inOff;
            int var10002 = len;
            System.arraycopy(in, inOff, this.buf, var5, var6);
            byte[] inOff1 = this.buf;
            byte[] len1 = this.mac;
            this.cipher.processBlock(inOff1, 0, len1, 0);
            this.bufOff = 0;
            inOff = var10002 - var6;

            for(len = var10000 + var6; inOff > var4; len += var4) {
               this.cipher.processBlock(in, len, this.mac, 0);
               inOff -= var4;
            }
         } else {
            var10000 = inOff;
            inOff = len;
            len = var10000;
         }

         CMac var10 = this;
         CMac var10001 = this;
         CMac var10004 = this;
         byte[] var7 = this.buf;
         System.arraycopy(in, len, var7, var10004.bufOff, inOff);
         var10.bufOff = var10001.bufOff + inOff;
      }
   }

   public int doFinal(byte[] out, int outOff) {
      int var3 = this.cipher.getBlockSize();
      byte[] var7;
      if (this.bufOff == var3) {
         var7 = this.Lu;
      } else {
         ISO7816d4Padding var8;
         ISO7816d4Padding var10001 = var8 = new ISO7816d4Padding;
         var8.<init>();
         var7 = this.buf;
         var10001.addPadding(var7, this.bufOff);
         var7 = this.Lu2;
      }

      byte[] var5;
      for(int var4 = 0; var4 < (var5 = this.mac).length; ++var4) {
         byte[] var10000 = this.buf;
         var10000[var4] ^= var7[var4];
      }

      this.cipher.processBlock(this.buf, 0, var5, 0);
      int var6 = this.macSize;
      System.arraycopy(this.mac, 0, out, outOff, var6);
      this.reset();
      return this.macSize;
   }

   public void reset() {
      byte[] var2;
      for(int var1 = 0; var1 < (var2 = this.buf).length; ++var1) {
         var2[var1] = 0;
      }

      this.bufOff = 0;
      this.cipher.reset();
   }
}
